#!/usr/bin/env ruby
# Copyright (C) 2007 Sylvain SARMEJEANNE

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details. 

module Scruby

# Dissector for Ethernet
class Ether<Layer

attr_accessor :dst, :src, :type

def init

  @protocol = 'Ethernet'
  @fields_desc =  [ MACField('dst', ETHERADDR_ANY),
                    MACField('src', ETHERADDR_ANY),
                    XShortEnumField('type', ETHERTYPE_IPv4, ETHERTYPE_ALL) ]
end

end

# Dissector for ARP
class ARP<Layer

attr_accessor :hwtype, :ptype, :hwlen, :plen, :op, :hwsrc, :psrc, :hwdst, :pdst

def init

  @protocol = 'ARP'
  @fields_desc = [ XShortEnumField('hwtype', ARPHWTYPE_ETHER, ARPHWTYPE_ALL),
                   XShortEnumField('ptype',  ETHERTYPE_IPv4, ETHERTYPE_ALL),
                   ByteEnumField('hwlen', ARPHWLEN_ETHER, ARPHWLEN_ALL),
                   ByteEnumField('plen', ARPPROTOLEN_IPv4, ARPPROTOLEN_ALL),
                   ShortEnumField('op', ARPTYPE_WHOAS, ARPTYPE_ALL),
                   MACField('hwsrc', ETHERADDR_ANY),
                   IPField('psrc','127.0.0.1'),
                   MACField('hwdst', ETHERADDR_ANY),
                   IPField('pdst', '127.0.0.1') ]
end

end

# Dissector for IPv4
class IP<Layer

attr_accessor :version, :ihl, :tos, :len, :id, :flags, :frag, :ttl,
:proto, :chksum, :src, :dst

def init
  @protocol = 'IPv4'
  @fields_desc = [ BitField('version', 4, 4),
                   BitField('ihl', 5, 4),
                   XByteField('tos', 0),
                   ShortField('len', 20),
                   XShortField('id', 0),
                   FlagsField('flags', 0, 3, IPFLAGS),
                   BitField('frag', 0, 13),
                   ByteField('ttl', 64),
                   ByteEnumField('proto', IPPROTO_TCP, IPPROTO_ALL),
                   XShortField('chksum', 0),
                   IPField('src', '127.0.0.1'),
                   IPField('dst', '127.0.0.1') ]
end

def pre_send(underlayer, payload)

  # Total length
  self.len = 20 + payload.length

  # Checksum
  self.chksum = 0
  self.chksum = Layer.checksum(self.to_net())
end

end

# Dissector for ICMP
class ICMP<Layer

attr_accessor :type, :code, :chksum, :id, :seq

def init
  @protocol = 'ICMP'
  @fields_desc = [ ByteEnumField('type', ICMPTYPE_ECHO_REQ, ICMPTYPE_ALL),
                   ByteField('code', 0),
                   XShortField('chksum', 0),
                   XShortField('id', 0),
                   XShortField('seq', 0) ]
end

def pre_send(underlayer, payload)
  # Checksum
  self.chksum = 0
  self.chksum = Layer.checksum(self.to_net() + payload)
end

end

# Dissector for Raw
class Raw<Layer

attr_accessor :load

def init
  @protocol = 'Raw data'
  @fields_desc = [ StrField('load', '') ]
end

end

# Dissector for TCP
class TCP<Layer

attr_accessor :sport, :dport, :seq, :ack, :dataofs, :reserved, :flags,
:window, :chksum, :urgptr

def init
  @protocol = 'TCP'
  @fields_desc = [ ShortField('sport', 1024),
                   ShortField('dport', 80),
                   IntField('seq', 0),
                   IntField('ack', 0),
                   BitField('dataofs', 5, 4),
                   BitField('reserved', 0, 4),
                   FlagsField('flags', 0x2, 8, TCPFLAGS),
                   ShortField('window', 8192),
                   XShortField('chksum', 0),
                   ShortField('urgptr', 0) ]
end

def pre_send(underlayer, payload)

  # To compute the TCP checksum, the IP underlayer is needed.
  # Otherwise, the chksum field is left equal to 0.
  if underlayer.is_a?(IP)

    # Getting IP addresses from the IPFields
    ip_src = underlayer.fields_desc[10].to_net(underlayer.fields_desc[10])
    ip_dst = underlayer.fields_desc[11].to_net(underlayer.fields_desc[11])

    this_packet = self.to_net()

    pseudo_header = [ip_src,
                     ip_dst,
                     underlayer.proto,
                     (this_packet + payload).length
                    ].pack("a4a4nn")

    self.chksum = 0
    self.chksum = Layer.checksum(pseudo_header + this_packet + payload)
  end
end

end

# Dissector for UDP
class UDP<Layer

attr_accessor :sport, :dport, :len, :chksum

def init
  @protocol = 'UDP'
  @fields_desc = [ ShortField('sport', 53),
                   ShortField('dport', 53),
                   ShortField('len', 8),
                   XShortField('chksum', 0) ]
end

# Almost the same as TCP
def pre_send(underlayer, payload)

  # Total length
  self.len = 8 + payload.length
  
  # To compute the UDP checksum, the IP underlayer is needed.
  # Otherwise, the chksum field is left equal to 0.
  if underlayer.is_a?(IP)

    # Getting IP addresses from the IPFields
    ip_src = underlayer.fields_desc[10].to_net(underlayer.fields_desc[10])
    ip_dst = underlayer.fields_desc[11].to_net(underlayer.fields_desc[11])

    this_packet = self.to_net()

    pseudo_header = [ip_src,
                     ip_dst,
                     underlayer.proto,
                     (this_packet + payload).length
                    ].pack("a4a4nn")
    
    self.chksum = 0
    self.chksum = Layer.checksum(pseudo_header + this_packet + payload)
  end

end

end

# Dissector for the classic BSD loopback header (NetBSD, FreeBSD and Mac OS X)
class ClassicBSDLoopback<Layer

attr_accessor :header

def init
  @protocol = 'Classic BSD loopback'
  @fields_desc = [ HostOrderIntField('header', BSDLOOPBACKTYPE_IPv4) ]
end

end

# Dissector for the OpenBSD loopback header
class OpenBSDLoopback<Layer

attr_accessor :header

def init
  @protocol = 'OpenBSD loopback'
  @fields_desc = [ LEIntField('header', BSDLOOPBACKTYPE_IPv4) ]
end

end

# Dissector for RIFF file format header
class RIFF<Layer

attr_accessor :id, :size, :headerid

def init
  @protocol = 'RIFF chunk'
  @fields_desc = [ StrFixedLenField('id', 'RIFF', 4),
                   LEIntField('size', 0),
                   StrFixedLenField('headerid', 'ACON', 4) ]
end

end

# Dissector for ANI header chunk  format
class ANI<Layer

attr_accessor :id, :size, :headersize, :frames, :steps, :width, :height, :bitcount, :planes, :displayrate, :reserved, :sequence, :icon

def init
  @protocol = 'ANI chunk'
  @fields_desc = [ StrFixedLenField('id', 'anih', 4),
                   LEIntField('size', 36),
                   LEIntField('headersize', 36),
                   LEIntField('frames', 2),
                   LEIntField('steps', 0),
                   LEIntField('width', 0),
                   LEIntField('height', 0),
                   LEIntField('bitcount', 0),
                   LEIntField('planes', 0),
                   LEIntField('displayrate', 0),
                   BitField('icon', 0, 1),
                   BitField('sequence', 0, 1),
                   BitField('reserved', 0, 30) ]
end

end

# Dot11 dissectors
class Dot11<Layer

attr_accessor :subtype, :type, :proto, :FCfield, :ID, :addr1, :addr2, :addr3, :SC, :addr4

def init
  @protocol = '802.11'
  @fields_desc = [ BitField('subtype', 0, 4),
                   BitEnumField('type', DOT11TYPE_DATA, 2, DOT11TYPE_ALL),
                   BitField('proto', 0, 2),
                   FlagsField('FCfield', 0, 8, DOT11_FC_FLAGS),
                   ShortField('ID', 0),
                   MACField('addr1', ETHERADDR_ANY),
                   Dot11Addr2MACField('addr2', ETHERADDR_ANY),
                   Dot11Addr3MACField('addr3', ETHERADDR_ANY),
                   Dot11SCField('SC', 0),
                   Dot11Addr4MACField('addr4', ETHERADDR_ANY) ]
end

end

class Dot11QoS<Layer

attr_accessor :TID, :EOSP, :AckPolicy, :Reserved, :TXOP

def init
  @protocol = '802.11 QoS'
  @fields_desc = [ BitField('TID',0,4),
                   BitField('EOSP',0,1),
                   BitField('AckPolicy',0,2),
                   BitField('Reserved',0,1),
                   ByteField('TXOP',0) ]
end

end

class Dot11Beacon<Layer

attr_accessor :timestamp, :beacon_interval, :cap

def init
  @protocol = '802.11 Beacon'
  @fields_desc = [ LongField('timestamp', 0), # Bug: should be little endian
                   LEShortField('beacon_interval', 0x64),
                   FlagsField('cap', 0, 16, DOT11_CAPABILITIES) ]
end

end

class Dot11Elt<Layer

attr_accessor :ID, :len, :info

def init
  @protocol = '802.11 Information Element'
  @fields_desc = [ ByteEnumField('ID', 0, DOT11_ID),
                   FieldLenField('len', 0, 'info', 'C'),
                   StrLenField('info', '', 'len') ]
end

end

class Dot11ATIM<Layer
def init
  @protocol = '802.11 ATIM'
end
end

class Dot11Disas<Layer

attr_accessor :reason

def init
  @protocol = '802.11 Disassociation'
  @fields_desc = [ LEShortEnumField('reason', 1, DOT11_REASON) ]
end

end

class Dot11AssoReq<Layer

attr_accessor :cap, :listen_interval

def init
  @protocol = '802.11 Association Request'
  @fields_desc = [ FlagsField('cap', 0, 16, DOT11_CAPABILITIES),
                   LEShortField('listen_interval', 0xc8) ]
end

end

class Dot11AssoResp<Layer

attr_accessor :cap, :status, :AID

def init
  @protocol = '802.11 Association Response'
  @fields_desc = [ FlagsField('cap', 0, 16, DOT11_CAPABILITIES),
                   LEShortField('status', 0),
                   LEShortField('AID', 0) ]
end

end

class Dot11ReassoReq<Layer

attr_accessor :cap, :current_AP, :listen_interval

def init
  @protocol = '802.11 Reassociation Request'
  @fields_desc = [ FlagsField('cap', 0, 16, DOT11_CAPABILITIES),
                   MACField('current_AP', ETHERADDR_ANY),
                   LEShortField('listen_interval', 0xc8) ]
end

end

class Dot11ReassoResp<Dot11AssoResp
def init
  @protocol = '802.11 Reassociation Response'
end
end

class Dot11ProbeReq<Layer
def init
  @protocol = '802.11 Probe Request'
end
end

class Dot11ProbeResp<Layer

attr_accessor :timestamp, :beacon_interval, :cap

def init
  @protocol = '802.11 Probe Response'
  @fields_desc = [ LongField('timestamp', 0), # Bug: should be little endian
                   LEShortField('beacon_interval', 0x64),
                   FlagsField('cap', 0, 16, DOT11_CAPABILITIES) ]
end

end

class Dot11Auth<Layer

attr_accessor :algo, :seqnum, :status

def init
  @protocol = '802.11 Authentication'
  @fields_desc = [ LEShortEnumField('algo', 0, DOT11_AUTH_ALGO),
                   LEShortField('seqnum', 0),
                   LEShortEnumField('status', 0, DOT11_STATUS) ]
end

end

class Dot11Deauth<Layer

attr_accessor :reason

def init
  @protocol = '802.11 Deauthentication'
  @fields_desc = [ LEShortEnumField('reason', 1, DOT11_REASON) ]
end

end

class Dot11WEP<Layer

attr_accessor :iv, :keyid, :wepdata, :icv

def init
  @protocol = '802.11 WEP packet'
  @fields_desc = [ StrFixedLenField('iv', "\0\0\0", 3),
                   ByteField('keyid', 0),
                   StrField('wepdata', ''), # Bug: 4 bytes remains
                   IntField('icv', 0) ]
end

end

class LLC<Layer

attr_accessor :dsap, :ssap, :ctrl

def init
  @protocol = 'LLC'
  @fields_desc = [ XByteField('dsap', 0),
                   XByteField('ssap', 0),
                   ByteField('ctrl', 0) ]
end

end

# Layer bounds
$layer_bounds =
{
'Ether' => [
            ['type', ETHERTYPE_IPv4, IP],
            ['type', ETHERTYPE_ARP, ARP]
           ],

'Dot11' => [
             ['type', 2, LLC],
             ['subtype', 0, Dot11AssoReq],
             ['subtype', 1, Dot11AssoResp],
             ['subtype', 2, Dot11ReassoReq],
             ['subtype', 3, Dot11ReassoResp],
             ['subtype', 4, Dot11ProbeReq],
             ['subtype', 5, Dot11ProbeResp],
             ['subtype', 8, Dot11Beacon],
             ['subtype', 9, Dot11ATIM],
             ['subtype', 10, Dot11Disas],
             ['subtype', 11, Dot11Auth],
             ['subtype', 12, Dot11Deauth],
            ],

'Dot11QoS' => [
                [BIND_ALWAYS, BIND_ALWAYS, LLC]
               ],
'Dot11Beacon' => [
                [BIND_ALWAYS, BIND_ALWAYS, Dot11Elt]
               ],
'Dot11AssoReq' => [
                [BIND_ALWAYS, BIND_ALWAYS, Dot11Elt]
               ],
'Dot11AssoResp' => [
                [BIND_ALWAYS, BIND_ALWAYS, Dot11Elt]
               ],
'Dot11ReassoReq' => [
                [BIND_ALWAYS, BIND_ALWAYS, Dot11Elt]
               ],
'Dot11ReassoResp' => [
                [BIND_ALWAYS, BIND_ALWAYS, Dot11Elt]
               ],
'Dot11ProbeReq' => [
                [BIND_ALWAYS, BIND_ALWAYS, Dot11Elt]
               ],
'Dot11ProbeResp' => [
                [BIND_ALWAYS, BIND_ALWAYS, Dot11Elt]
               ],
'Dot11Auth' => [
                [BIND_ALWAYS, BIND_ALWAYS, Dot11Elt]
               ],
'Dot11Elt' => [
                [BIND_ALWAYS, BIND_ALWAYS, Dot11Elt]
               ],

'ClassicBSDLoopback' => [
                         ['header', BSDLOOPBACKTYPE_IPv4, IP]
                        ],

'OpenBSDLoopback' => [
                      ['header', BSDLOOPBACKTYPE_IPv4, IP]
                     ],

'IP' => [
         ['proto', IPPROTO_ICMP, ICMP],
         ['proto', IPPROTO_TCP, TCP],
         ['proto', IPPROTO_UDP, UDP]
        ],
    }

# One day, this will be processed automatically :)
DISSECTORS_LIST = [Ether, ARP, IP, ICMP, Raw, TCP, UDP, ClassicBSDLoopback,
                  OpenBSDLoopback, RIFF, ANI,
                  Dot11, Dot11QoS, Dot11Beacon, Dot11Elt, Dot11ATIM, Dot11Disas,
                  Dot11AssoReq, Dot11AssoResp, Dot11ReassoReq, Dot11ReassoResp,
                  Dot11ProbeReq, Dot11ProbeResp, Dot11Auth, Dot11Deauth, Dot11WEP,
                  LLC]
end
